package com.atomikos.demo.jaxws.server;

import java.util.List;

import javax.xml.ws.Endpoint;
import javax.xml.ws.handler.Handler;

import com.atomikos.demo.jaxws.impl.WebShop;
import com.atomikos.icatch.jta.UserTransactionManager;
import com.atomikos.icatch.jaxws.wsat.ImportingTransactionHandler;

// for the atomikos commit protocol (not WSAT) use the following instead:
// import com.atomikos.icatch.jaxws.atomikos.ImportingTransactionHandler;

public class Server {

	public static void main(String[] args) throws Exception {
		UserTransactionManager utm = new UserTransactionManager();
		utm.init();

		Endpoint endpoint = Endpoint.create(new WebShop());
		List<Handler> chain = endpoint.getBinding().getHandlerChain();
		
		//incoming requests are intercepted by an 
		//Atomikos handler to deal with transaction aspects
		ImportingTransactionHandler handler = new ImportingTransactionHandler();
		//depending on the importPreference, 
		//a new tx may be created for the incoming call
		//just like in regular JEE
		handler.setImportPreference("Required");
		//new transactions time out when? (milliseconds)
		handler.setNewTransactionTimeout ( 10000 );

		//should newly created transactions be JTA-like or TCC-like?
                //UNCOMMMENT for the ATOMIKOS commit protocol (not wsat) 
		//handler.setJtaCompatible(true);

		//when a heuristic timeout happens, commit or rollback?
		handler.setCommitOnHeuristicTimeout(true);
		
		chain.add(handler);
		
		endpoint.getBinding().setHandlerChain(chain);
		endpoint.publish("http://0.0.0.0:8888/shop");
	    
		System.out.println("server started");
		
		while (true) {
			Thread.sleep(1000);
			if (1 == 2)
				break;
		}
		
		utm.close();
	}

}
