package tcc.payment;

import java.rmi.Remote;
import java.rmi.RemoteException;

import com.atomikos.icatch.Extent;
import com.atomikos.icatch.Propagation;

 /**
  *Copyright &Copy; 2006, Atomikos. All rights reserved.
  *
  *An RMI payment server that accepts payment orders within the 
  *scope of a remote TCC transaction.
  *
  *The purpose of this interface is to show the essentials
  *of TCC and transaction propagation: a remote transaction is 
  *imported for each invocation, and the <b>extent</b> of the local
  *work is returned to the remote party (for two-phase commit
  *processing).
  */

public interface PaymentServer
extends Remote
{
     /**
      *Process a payment for the given card.
      *
      *@param propagation The propagation data of the invoker's
      *transaction. This indicates that the service is only to be invoked
      *from within an existing remote transaction.
      *@param cardno The credit card number to use.
      *@param amount The amount to be charged.
      *@return Extent The transaction-service related data that 
      *will be used at the invoker's side to process two-phase
      *commit termination for the invocation.
      *@exception PaymentServerException On payment errors, such as 
      *unknown card.
      *
      *@exception RemoteException Required by RMI.
      */
    
    public Extent pay ( Propagation propagation, String cardno , int amount ) 
    throws PaymentServerException, RemoteException;
    
     /**
      *Retrieve all payments done for a given card number.
      *
      *@param cardno The card number to query for.
      *@return int The overall total purchased on the card.
      *@exception PaymentServerException For payment errors, such as 
      *card unknown.
      *@exception RemoteException Required by RMI.
      */
      
    public int getPayments ( String cardno )
    throws PaymentServerException , RemoteException;
}
