package tcc.portal;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;

import javax.naming.Context;
import javax.naming.InitialContext;

import portal.PortalDbUtils;
import portal.PortalTccService;
import tcc.payment.PaymentServer;

import com.atomikos.icatch.ExportingTransactionManager;
import com.atomikos.icatch.Extent;
import com.atomikos.icatch.Propagation;
import com.atomikos.icatch.config.UserTransactionService;
import com.atomikos.icatch.config.UserTransactionServiceImp;
import com.atomikos.icatch.tcc.UserTccServiceManager;




 /**
  *Copyright &copy; 2004, Atomikos. All rights reserved.
  *
  *An implementation of the portal server.
  *This example shows how to set up TransactionsRMI
  *and how to implement invocations that are either 
  *separate transactions or all part of one local transaction
  *with client-demarcation.
  */
  
public class PortalServerImp
implements PortalServer
{
   
    
    private static String paymentServer = "";
    
    private static UserTransactionService uts;
    
    private static UserTccServiceManager tccMgr;
    //the instance to start/terminate TCC transactions
    
    private static PortalTccService service;
    //the actual TCC service to invoke
      
      /**
        *Initialize the service, and setup DB tables if needed.
        */
        
      private void setup()
      throws Exception
      {
    	  	
    	  		tccMgr = new UserTccServiceManager();
    	  	    service = new PortalTccService ( tccMgr , 1000 );
    	  	    tccMgr.registerForRecovery ( service );
    	  	    uts = new UserTransactionServiceImp();
    	  	    uts.init ( uts.createTSInitInfo() );
    	  	    
             PortalDbUtils.createDatabase();
             
            //Export the server into RMI
            UnicastRemoteObject.exportObject ( this );
            //bind user transaction in JNDI
            
            Context ctx = new InitialContext ();

            
            ctx.rebind ( "PortalServer" , this );
            
            
            //That concludes setup
           
      }
      
      /**
        *Shutdown the TM and the datasource.
        */
        
      private  void shutdown()
      throws Exception
      {
	  	    //shutdown the transaction service 
	  		//but wait for active transactions
	  		uts.shutdown ( false );
            
            UnicastRemoteObject.unexportObject ( this , false );
                        
            //unbind usertx from JNDI
           
            Context ctx = new InitialContext();
            ctx.unbind ( "PortalServer" );
           

      }
   
      
      private static ExportingTransactionManager 
      getExportingTransactionManager()
      {
          return uts.getExportingTransactionManager(); 
      }
      
     
     /**
      *@see PortalServer
      */
      
    public void purchase ( String cardno , int itemId , int quantity )
    throws PortalServerException, RemoteException
    {
        boolean error = false;
        //true ASA exception 
        ExportingTransactionManager exptm =
            getExportingTransactionManager();
        Connection conn = null;
        Propagation propagation = null;
        Extent extent = null;
        String id = null;
        try {
        	   
        	   //create a top-level activity to relate the outcome
        	   //of both the purchase and payment operations
        	   id = tccMgr.register ( null , 10000 );
        	
        	   //perform the local try-phase of the work 
        	   int price = service.tryPurchase ( itemId , quantity );
            
            //now, set up to delegate payment
            //to do so, get a propagation for the transaction of the current 
            //thread.
            propagation = exptm.getPropagation();

            Context ctx = new InitialContext();
            
            //lookup the payment server
            PaymentServer pserver = 
                ( PaymentServer ) ctx.lookup ( paymentServer );
            extent = pserver.pay ( propagation , cardno , quantity*price );
            
            //add the two-phase commit information after the call is done.
            //this allows our transaction service to contact the remote 
            //for two-phase commit
            //NOTE: if an exception happens during the call then the
            //remote will not receive two-phase commit; this is 
            //perfectly OK; it allows us to tolerate remote failures
            //and still try an alternative.
            exptm.addExtent ( extent );
            
            tccMgr.completed ( id );
            
            
        }
        catch ( Exception e ) {
              //e.printStackTrace();
        	     tccMgr.failed ( id );
              error = true; 
              throw new PortalServerException ( e.getMessage() );
          }

        
    }
    
  

      
      /**
       *@see PortalServer
       */
       
    public int getAmountPurchased ( String cardno )
    throws PortalServerException, RemoteException
    {
        boolean error = false;
        //true ASA exception 
        int res = 0;
        //the result
        
       
        try {
 
            Context ctx = new InitialContext();
            //lookup the payment server
            PaymentServer pserver = 
                ( PaymentServer ) ctx.lookup ( paymentServer );
            res = pserver.getPayments ( cardno );
                                  
        }
        catch ( Exception e ) {
              //e.printStackTrace();
              error = true; 
              throw new PortalServerException ( e.getMessage() );
          }
          
          
          return res;
    }
    
     /**
      *@see PortalServer
      */
      
    public int getStock(int itemId) throws PortalServerException,
			RemoteException {
		int ret = 0;
		try {
			ret = service.getStock(itemId);
		} catch (Exception e) {
			throw new PortalServerException();
		}
		return ret;
	}
    
 
    public static void main ( String[] args ) 
    {
        BufferedReader breader = null;
        
        if ( args.length !=1 ) {
            System.err.println ( 
                "Argument required: RMI URL of payment server." );
            System.exit ( 1 );
        }
        
        
        
        try {
            paymentServer = args[0];
                
            PortalServerImp server = new PortalServerImp();
            server.setup();
            
            System.out.println ( "Server running..." );
            System.out.println ( "Type ENTER to shutdown." );
            
            breader = 
              new BufferedReader ( new InputStreamReader ( System.in ) );
            breader.readLine();
            
            server.shutdown();

            System.exit ( 0 );
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }
}

